@@ -17,13 +17,13 @@ from pika.exchange_type import ExchangeType
 from pika.frame import Method
 from pika.spec import Basic
 
-__all__ = ('RabbitApp',)
+__all__ = ('RabbitClient',)
 log = logging.getLogger('snacks')
 
 
 # TODO django_rest_framework serializer support?
 # TODO default request/response serializers?
-class RabbitApp:
+class RabbitClient:
     """A class to interface with RabbitMQ."""
 
     def __init__(
@@ -60,14 +60,15 @@ class RabbitApp:
         self.retry_timeout: float = retry_timeout or 10.0
 
     @staticmethod
-    def from_dict(dictionary: dict[str, Any]) -> 'RabbitApp':
-        return RabbitApp(
+    def from_dict(dictionary: dict[str, Any]) -> 'RabbitClient':
+        return RabbitClient(
             dictionary.get('host'),
             dictionary.get('port'),
             dictionary.get('exchange'),
             dictionary.get('virtual_host'),
             dictionary.get('username') or dictionary.get('user') or 'guest',
-            dictionary.get('password') or dictionary.get('pass') or 'guest'
+            dictionary.get('password') or dictionary.get('pass') or 'guest',
+            dictionary.get('retry_timeout') or 10.0
         )
 
     def exchange_declare(
@@ -291,7 +292,8 @@ class RabbitApp:
         The decorated function can have as parameters, any or all of,
         message body, method, and properties.
 
-        A new channel and thread is created for each consumer.
+        A new connection, channel, and thread is created for each
+        consumer.
 
         :param routing_keys: Key or keys to generate queues for.
         :param exchange: The queues to consume from.
@@ -305,20 +307,27 @@ class RabbitApp:
 
         # If a connection is lost, the auto-deleting queues will all be
         # gone so reconnecting won't work. We pass the consume logic
-        # this queue_factory to generate the initially and whenever a
-        # connection fails.
-        def queue_factory() -> list[str]:
+        # this queue_factory to generate the queues initially and
+        # whenever a connection fails.
+        def queue_factory(channel: BlockingChannel) -> list[str]:
             queues = []
             for key in rks:
-                q = self.queue_declare(
-                    arguments=declare_arguments,
-                    auto_delete=True
+                log.debug('Declaring new queue top bind to [%s].', key)
+                q = channel.queue_declare(
+                    queue='',
+                    auto_delete=True,
+                    arguments=declare_arguments
                 ).method.queue
-                self.queue_bind(q, key, exchange, arguments=bind_arguments)
+                channel.queue_bind(q, exchange, key, arguments=bind_arguments)
                 queues.append(q)
+                log.debug('[%s] bound to [%s] with [%s]', q, exchange, key)
             return queues
 
         def wrapper(fun: Callable) -> Any:
+            if fun.__name__ == '<lambda>':
+                fun_name = f'{"_".join(rks)}_lambda_listener'.replace('.', '_')
+                log.debug('Renaming lambda to [%s]', fun_name)
+                fun.__name__ = fun_name
             thread = threading.Thread(
                 target=self._consume,
                 args=(fun, [], serialize, queue_factory)
@@ -326,7 +335,12 @@ class RabbitApp:
             thread.daemon = True
             thread.start()
             for rk in rks:
-                log.info('[%s] consuming queue for key [%s]', fun.__name__, rk)
+                log.info(
+                    'Thread spawned for [%s] to consume a queue on [%s - %s]',
+                    fun.__name__,
+                    exchange,
+                    rk
+                )
 
         return wrapper
 
@@ -340,7 +354,8 @@ class RabbitApp:
         The decorated function can have as parameters, any or all of,
         message body, method, and properties.
 
-        A new channel and thread is created for each consumer.
+        A new connection, channel, and thread is created for each
+        consumer.
 
         :param queues: The queue or queues to consume from.
         :param serialize: Callable to serialize message bodies.
@@ -362,37 +377,42 @@ class RabbitApp:
             fun: Callable,
             queues: Union[Iterable[str], str],
             serialize: Optional[Callable] = None,
-            queue_factory: Optional[Callable[[], list[str]]] = None
+            q_factory: Optional[Callable[[BlockingChannel], list[str]]] = None
     ) -> None:
         running = True
         channel = None
+        connection = None
         while running:
             try:
-                if self.connection.is_closed:
-                    self._reset_if_lost()
+                # New channel and connection since threads can't share.
+                if not connection or connection.is_closed:
+                    connection = BlockingConnection(self.params)
                 if not channel or channel.is_closed:
-                    # New channel since threads can't share channel.
-                    channel = self.connection.channel()
+                    channel = connection.channel()
 
                 def consume(q: str) -> None:
                     _listen(channel, q, fun, serialize)
                     level = log.debug if q.startswith('amq.gen') else log.info
                     level('[%s] consuming queue [%s]', fun.__name__, q)
 
-                if queue_factory:
-                    queues = queue_factory()
+                if q_factory:
+                    queues = q_factory(channel)
                 queues = [queues] if isinstance(queues, str) else queues
                 [consume(queue) for queue in queues]
                 channel.start_consuming()
             except (ChannelClosed, AMQPChannelError, AMQPConnectionError) as e:
-                log.error('%s on channel for %s', type(e).__name__, queues)
+                log.error(
+                    '[%s] on consumer [%s]',
+                    type(e).__name__,
+                    fun.__name__
+                )
                 log.info(
                     'Retrying connection in [%s] seconds...',
                     self.retry_timeout
                 )
             except KeyboardInterrupt:
                 running = False
-                if channel:
+                if channel and channel.is_open:
                     channel.stop_consuming()
                     channel.close()
             time.sleep(self.retry_timeout)
